//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// create our OSC receiver
OscIn oscin;
// a thing to retrieve message contents
OscMsg msg;
// use port 12000 (default Wekinator output port)
12000 => oscin.port;

// expecting 1 output dimensions
5 => int NUM_PARAMS;
float myParams[NUM_PARAMS];

// listen for "/wek/output" message with 5 floats coming in
oscin.addAddress( "/wek/outputs, fffff" );
// print
<<< "listening for OSC message from Wekinator on port 12000...", "" >>>;
<<< " |- expecting \"/wek/outputs\" with 5 continuous parameters...", "" >>>;

Blit s => ADSR e => JCRev r => dac;
.5 => s.gain;
.05 => r.mix;

// set adsr
e.set( 5::ms, 3::ms, .5, 5::ms );

// an array
[ 0, 2, 4, 7, 9, 11 ] @=> int hi[];

fun void setParams( float params[] )
{
    // make sure we have enough
    if( params.size() >= NUM_PARAMS )
    {		
        // adjust the synthesis accordingly
        0.0 => float x;
        for( 0 => int i; i < NUM_PARAMS; i++ )
        {
            // get value
            params[i] => x;
            // clamp it
            if( x < 0 ) 0 => x;
            if( x > 1 ) 1 => x;
            // remember
            x => myParams[i];
        }
    }
}


fun void waitForEvent()
{
    // array to hold params
    float p[NUM_PARAMS];

    // infinite event loop
    while( true )
    {
        // wait for OSC message to arrive
        oscin => now;

        // grab the next message from the queue. 
        while( oscin.recv(msg) )
        {
            // print stuff
            cherr <= msg.address <= " ";
            
            // unpack our 5 floats into our array p
            for( int i; i < NUM_PARAMS; i++ )
            {
                // put into array
                msg.getFloat(i) => p[i];
                // print
                cherr <= p[i] <= " ";
            }
            
            // print
            cherr <= IO.newline();
            
            // set the parameters
            setParams( p );
        }
    }
}

// spork osc receiver loop
spork ~waitForEvent();

// infinite time loop
while( true )
{
    // frequency
    Std.mtof( 33 + Std.ftoi(myParams[0] * 5) * 12 +
        hi[Math.random2(0,hi.size()-1)] ) => s.freq;
    
    // Std.mtof( 33 + Math.random2(0,3) * 12 +
        // hi[Math.random2(0,hi.size()-1)] ) => s.freq;

    // harmonics
    Std.ftoi(myParams[1] * 4) + 1 => s.harmonics;
    // Math.random2( 1, 5 ) => s.harmonics;

    myParams[2]::ms * 200 => e.attackTime;
    myParams[3]::ms * 200 => e.decayTime;
    myParams[4]::ms * 200 => e.releaseTime;
    // e.set( 5::ms, 3::ms, .5, 5::ms );

    // key on
    e.keyOn();
    // advance time
    150::ms => now;
    // key off
    e.keyOff();
    // advance time
    5::ms => now;
}

// // spork mapping function
// spork ~ map2sound();	

// e.keyOn();

// 120::ms => now;

// e.keyOff();

// // time loop to keep everything going
// while( true ) 1::second => now;

